package java.awt.datatransfer;

import java.io.OptionalDataException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.List;
import java.io.Serializable;
import java.util.Objects;
import java.io.IOException;
import java.io.StringReader;
import java.io.CharArrayReader;
import java.io.InputStreamReader;
import java.io.ByteArrayInputStream;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.io.Reader;
import java.util.Collections;
import java.util.Arrays;
import sun.awt.datatransfer.DataTransferer;
import sun.security.util.SecurityConstants;
import sun.reflect.misc.ReflectUtil;
import java.util.Comparator;
import java.io.InputStream;
import java.io.Externalizable;

public class DataFlavor implements Externalizable, Cloneable
{
    private static final long serialVersionUID = 8367026044764648243L;
    private static final Class<InputStream> ioInputStreamClass;
    public static final DataFlavor stringFlavor;
    public static final DataFlavor imageFlavor;
    @Deprecated
    public static final DataFlavor plainTextFlavor;
    public static final String javaSerializedObjectMimeType = "application/x-java-serialized-object";
    public static final DataFlavor javaFileListFlavor;
    public static final String javaJVMLocalObjectMimeType = "application/x-java-jvm-local-objectref";
    public static final String javaRemoteObjectMimeType = "application/x-java-remote-object";
    public static DataFlavor selectionHtmlFlavor;
    public static DataFlavor fragmentHtmlFlavor;
    public static DataFlavor allHtmlFlavor;
    private static Comparator<DataFlavor> textFlavorComparator;
    transient int atom;
    MimeType mimeType;
    private String humanPresentableName;
    private Class<?> representationClass;
    
    protected static final Class<?> tryToLoadClass(String s, ClassLoader loader) throws ClassNotFoundException {
        ReflectUtil.checkPackageAccess(s);
        try {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkPermission(SecurityConstants.GET_CLASSLOADER_PERMISSION);
            }
            ClassLoader systemClassLoader = ClassLoader.getSystemClassLoader();
            try {
                return Class.forName(s, true, systemClassLoader);
            }
            catch (ClassNotFoundException ex) {
                ClassLoader contextClassLoader = Thread.currentThread().getContextClassLoader();
                if (contextClassLoader != null) {
                    try {
                        return Class.forName(s, true, contextClassLoader);
                    }
                    catch (ClassNotFoundException ex2) {}
                }
            }
        }
        catch (SecurityException ex3) {}
        return Class.forName(s, true, loader);
    }
    
    private static DataFlavor createConstant(Class<?> clazz, String s) {
        try {
            return new DataFlavor(clazz, s);
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    private static DataFlavor createConstant(String s, String s2) {
        try {
            return new DataFlavor(s, s2);
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    private static DataFlavor initHtmlDataFlavor(String str) {
        try {
            return new DataFlavor("text/html; class=java.lang.String;document=" + str + ";charset=Unicode");
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    public DataFlavor() {
    }
    
    private DataFlavor(String str, String str2, MimeTypeParameterList list, Class<?> representationClass, String humanPresentableName) {
        if (str == null) {
            throw new NullPointerException("primaryType");
        }
        if (str2 == null) {
            throw new NullPointerException("subType");
        }
        if (representationClass == null) {
            throw new NullPointerException("representationClass");
        }
        if (list == null) {
            list = new MimeTypeParameterList();
        }
        list.set("class", representationClass.getName());
        if (humanPresentableName == null) {
            humanPresentableName = list.get("humanPresentableName");
            if (humanPresentableName == null) {
                humanPresentableName = str + "/" + str2;
            }
        }
        try {
            this.mimeType = new MimeType(str, str2, list);
        }
        catch (MimeTypeParseException ex) {
            throw new IllegalArgumentException("MimeType Parse Exception: " + ex.getMessage());
        }
        this.representationClass = representationClass;
        this.humanPresentableName = humanPresentableName;
        this.mimeType.removeParameter("humanPresentableName");
        return;
    }
    
    public DataFlavor(Class<?> clazz, String s) {
        this("application", "x-java-serialized-object", null, clazz, s);
        if (clazz == null) {
            throw new NullPointerException("representationClass");
        }
        return;
    }
    
    public DataFlavor(String str, String s) {
        if (str == null) {
            throw new NullPointerException("mimeType");
        }
        try {
            this.initialize(str, s, this.getClass().getClassLoader());
        }
        catch (MimeTypeParseException ex) {
            throw new IllegalArgumentException("failed to parse:" + str);
        }
        catch (ClassNotFoundException ex2) {
            throw new IllegalArgumentException("can't find specified class: " + ex2.getMessage());
        }
        return;
    }
    
    public DataFlavor(String str, String s, ClassLoader classLoader) throws ClassNotFoundException {
        if (str == null) {
            throw new NullPointerException("mimeType");
        }
        try {
            this.initialize(str, s, classLoader);
        }
        catch (MimeTypeParseException ex) {
            throw new IllegalArgumentException("failed to parse:" + str);
        }
        return;
    }
    
    public DataFlavor(String str) throws ClassNotFoundException {
        if (str == null) {
            throw new NullPointerException("mimeType");
        }
        try {
            this.initialize(str, null, this.getClass().getClassLoader());
        }
        catch (MimeTypeParseException ex) {
            throw new IllegalArgumentException("failed to parse:" + str);
        }
        return;
    }
    
    private void initialize(String str, String humanPresentableName, ClassLoader classLoader) throws MimeTypeParseException, ClassNotFoundException {
        if (str == null) {
            throw new NullPointerException("mimeType");
        }
        this.mimeType = new MimeType(str);
        String parameter = this.getParameter("class");
        if (parameter == null) {
            if ("application/x-java-serialized-object".equals(this.mimeType.getBaseType())) {
                throw new IllegalArgumentException("no representation class specified for:" + str);
            }
            this.representationClass = InputStream.class;
        }
        else {
            this.representationClass = tryToLoadClass(parameter, classLoader);
        }
        this.mimeType.setParameter("class", this.representationClass.getName());
        if (humanPresentableName == null) {
            humanPresentableName = this.mimeType.getParameter("humanPresentableName");
            if (humanPresentableName == null) {
                humanPresentableName = this.mimeType.getPrimaryType() + "/" + this.mimeType.getSubType();
            }
        }
        this.humanPresentableName = humanPresentableName;
        this.mimeType.removeParameter("humanPresentableName");
        return;
    }
    
    @Override
    public String toString() {
        return this.getClass().getName() + "[" + this.paramString() + "]";
    }
    
    private String paramString() {
        String string = "" + "mimetype=";
        String str;
        if (this.mimeType == null) {
            str = string + "null";
        }
        else {
            str = string + this.mimeType.getBaseType();
        }
        String string2 = str + ";representationclass=";
        String str2;
        if (this.representationClass == null) {
            str2 = string2 + "null";
        }
        else {
            str2 = string2 + this.representationClass.getName();
        }
        if (DataTransferer.isFlavorCharsetTextType(this) && (this.isRepresentationClassInputStream() || this.isRepresentationClassByteBuffer() || byte[].class.equals(this.representationClass))) {
            str2 = str2 + ";charset=" + DataTransferer.getTextCharset(this);
            return str2;
        }
        return str2;
    }
    
    public static final DataFlavor getTextPlainUnicodeFlavor() {
        String defaultUnicodeEncoding = null;
        DataTransferer instance = DataTransferer.getInstance();
        if (instance != null) {
            defaultUnicodeEncoding = instance.getDefaultUnicodeEncoding();
        }
        return new DataFlavor("text/plain;charset=" + defaultUnicodeEncoding + ";class=java.io.InputStream", "Plain Text");
    }
    
    public static final DataFlavor selectBestTextFlavor(DataFlavor[] a) {
        if (a == null || a.length == 0) {
            return null;
        }
        if (DataFlavor.textFlavorComparator == null) {
            DataFlavor.textFlavorComparator = new DataFlavor.TextFlavorComparator();
        }
        DataFlavor dataFlavor = (DataFlavor)Collections.max(Arrays.asList(a), DataFlavor.textFlavorComparator);
        if (!dataFlavor.isFlavorTextType()) {
            return null;
        }
        return dataFlavor;
    }
    
    public Reader getReaderForText(Transferable transferable) throws UnsupportedFlavorException, IOException {
        Object transferData = transferable.getTransferData(this);
        if (transferData == null) {
            throw new IllegalArgumentException("getTransferData() returned null");
        }
        if ((transferData instanceof Reader)) {
            return (Reader)transferData;
        }
        if ((transferData instanceof String)) {
            return new StringReader((String)transferData);
        }
        if ((transferData instanceof CharBuffer)) {
            CharBuffer charBuffer = (CharBuffer)transferData;
            int remaining2 = charBuffer.remaining();
            char[] array2 = new char[remaining2];
            charBuffer.get(array2, 0, remaining2);
            return new CharArrayReader(array2);
        }
        if ((transferData instanceof char[])) {
            return new CharArrayReader((char[])(char[])transferData);
        }
        InputStream inputStream = null;
        if ((transferData instanceof InputStream)) {
            inputStream = (InputStream)transferData;
        }
        else if ((transferData instanceof ByteBuffer)) {
            ByteBuffer byteBuffer = (ByteBuffer)transferData;
            int remaining = byteBuffer.remaining();
            byte[] array = new byte[remaining];
            byteBuffer.get(array, 0, remaining);
            inputStream = new ByteArrayInputStream(array);
        }
        else if ((transferData instanceof byte[])) {
            inputStream = new ByteArrayInputStream((byte[])(byte[])transferData);
        }
        if (inputStream == null) {
            throw new IllegalArgumentException("transfer data is not Reader, String, CharBuffer, char array, InputStream, ByteBuffer, or byte array");
        }
        String parameter = this.getParameter("charset");
        return (parameter == null) ? new InputStreamReader(inputStream) : new InputStreamReader(inputStream, parameter);
    }
    
    public String getMimeType() {
        return (this.mimeType != null) ? this.mimeType.toString() : null;
    }
    
    public Class<?> getRepresentationClass() {
        return this.representationClass;
    }
    
    public String getHumanPresentableName() {
        return this.humanPresentableName;
    }
    
    public String getPrimaryType() {
        return (this.mimeType != null) ? this.mimeType.getPrimaryType() : null;
    }
    
    public String getSubType() {
        return (this.mimeType != null) ? this.mimeType.getSubType() : null;
    }
    
    public String getParameter(String s) {
        if (s.equals("humanPresentableName")) {
            return this.humanPresentableName;
        }
        return (this.mimeType != null) ? this.mimeType.getParameter(s) : null;
    }
    
    public void setHumanPresentableName(String humanPresentableName) {
        this.humanPresentableName = humanPresentableName;
    }
    
    @Override
    public boolean equals(Object o) {
        return o instanceof DataFlavor && this.equals((DataFlavor)o);
    }
    
    public boolean equals(DataFlavor dataFlavor) {
        if (dataFlavor == null) {
            return false;
        }
        if (this == dataFlavor) {
            return true;
        }
        if (!Objects.equals(this.getRepresentationClass(), dataFlavor.getRepresentationClass())) {
            return false;
        }
        if (this.mimeType == null) {
            if (dataFlavor.mimeType != null) {
                return false;
            }
        }
        else {
            if (!this.mimeType.match(dataFlavor.mimeType)) {
                return false;
            }
            if ("text".equals(this.getPrimaryType())) {
                if (DataTransferer.doesSubtypeSupportCharset(this) && this.representationClass != null && !this.isStandardTextRepresentationClass() && !Objects.equals(DataTransferer.canonicalName(this.getParameter("charset")), DataTransferer.canonicalName(dataFlavor.getParameter("charset")))) {
                    return false;
                }
                if ("html".equals(this.getSubType()) && !Objects.equals(this.getParameter("document"), dataFlavor.getParameter("document"))) {
                    return false;
                }
            }
        }
        return true;
    }
    
    @Deprecated
    public boolean equals(String s) {
        return s != null && this.mimeType != null && this.isMimeTypeEqual(s);
    }
    
    @Override
    public int hashCode() {
        int n = 0;
        if (this.representationClass != null) {
            n += this.representationClass.hashCode();
        }
        if (this.mimeType != null) {
            String primaryType = this.mimeType.getPrimaryType();
            if (primaryType != null) {
                n += primaryType.hashCode();
            }
            if ("text".equals(primaryType)) {
                if (DataTransferer.doesSubtypeSupportCharset(this) && this.representationClass != null && !this.isStandardTextRepresentationClass()) {
                    String canonicalName = DataTransferer.canonicalName(this.getParameter("charset"));
                    if (canonicalName != null) {
                        n += canonicalName.hashCode();
                    }
                }
                if ("html".equals(this.getSubType())) {
                    String parameter = this.getParameter("document");
                    if (parameter != null) {
                        n += parameter.hashCode();
                    }
                }
            }
        }
        return n;
    }
    
    public boolean match(DataFlavor dataFlavor) {
        return this.equals(dataFlavor);
    }
    
    public boolean isMimeTypeEqual(String s) {
        if (s == null) {
            throw new NullPointerException("mimeType");
        }
        if (this.mimeType == null) {
            return false;
        }
        try {
            return this.mimeType.match(new MimeType(s));
        }
        catch (MimeTypeParseException ex) {
            return false;
        }
    }
    
    public final boolean isMimeTypeEqual(DataFlavor dataFlavor) {
        return this.isMimeTypeEqual(dataFlavor.mimeType);
    }
    
    private boolean isMimeTypeEqual(MimeType mimeType) {
        if (this.mimeType == null) {
            return mimeType == null;
        }
        return this.mimeType.match(mimeType);
    }
    
    private boolean isStandardTextRepresentationClass() {
        return this.isRepresentationClassReader() || String.class.equals(this.representationClass) || this.isRepresentationClassCharBuffer() || char[].class.equals(this.representationClass);
    }
    
    public boolean isMimeTypeSerializedObject() {
        return this.isMimeTypeEqual("application/x-java-serialized-object");
    }
    
    public final Class<?> getDefaultRepresentationClass() {
        return DataFlavor.ioInputStreamClass;
    }
    
    public final String getDefaultRepresentationClassAsString() {
        return this.getDefaultRepresentationClass().getName();
    }
    
    public boolean isRepresentationClassInputStream() {
        return DataFlavor.ioInputStreamClass.isAssignableFrom(this.representationClass);
    }
    
    public boolean isRepresentationClassReader() {
        return Reader.class.isAssignableFrom(this.representationClass);
    }
    
    public boolean isRepresentationClassCharBuffer() {
        return CharBuffer.class.isAssignableFrom(this.representationClass);
    }
    
    public boolean isRepresentationClassByteBuffer() {
        return ByteBuffer.class.isAssignableFrom(this.representationClass);
    }
    
    public boolean isRepresentationClassSerializable() {
        return Serializable.class.isAssignableFrom(this.representationClass);
    }
    
    public boolean isRepresentationClassRemote() {
        return DataTransferer.isRemote(this.representationClass);
    }
    
    public boolean isFlavorSerializedObjectType() {
        return this.isRepresentationClassSerializable() && this.isMimeTypeEqual("application/x-java-serialized-object");
    }
    
    public boolean isFlavorRemoteObjectType() {
        return this.isRepresentationClassRemote() && this.isRepresentationClassSerializable() && this.isMimeTypeEqual("application/x-java-remote-object");
    }
    
    public boolean isFlavorJavaFileListType() {
        return this.mimeType != null && this.representationClass != null && List.class.isAssignableFrom(this.representationClass) && this.mimeType.match(DataFlavor.javaFileListFlavor.mimeType);
    }
    
    public boolean isFlavorTextType() {
        return DataTransferer.isFlavorCharsetTextType(this) || DataTransferer.isFlavorNoncharsetTextType(this);
    }
    
    @Override
    public synchronized void writeExternal(ObjectOutput objectOutput) throws IOException {
        if (this.mimeType != null) {
            this.mimeType.setParameter("humanPresentableName", this.humanPresentableName);
            objectOutput.writeObject(this.mimeType);
            this.mimeType.removeParameter("humanPresentableName");
        }
        else {
            objectOutput.writeObject(null);
        }
        objectOutput.writeObject(this.representationClass);
    }
    
    @Override
    public synchronized void readExternal(ObjectInput objectInput) throws IOException, ClassNotFoundException {
        String parameter = null;
        this.mimeType = (MimeType)objectInput.readObject();
        if (this.mimeType != null) {
            this.humanPresentableName = this.mimeType.getParameter("humanPresentableName");
            this.mimeType.removeParameter("humanPresentableName");
            parameter = this.mimeType.getParameter("class");
            if (parameter == null) {
                throw new IOException("no class parameter specified in: " + this.mimeType);
            }
        }
        try {
            this.representationClass = (Class)objectInput.readObject();
        }
        catch (OptionalDataException ex) {
            if (!ex.eof || ex.length != 0) {
                throw ex;
            }
            if (parameter != null) {
                this.representationClass = tryToLoadClass(parameter, this.getClass().getClassLoader());
            }
        }
    }
    
    public Object clone() throws CloneNotSupportedException {
        Object clone = super.clone();
        if (this.mimeType != null) {
            ((DataFlavor)clone).mimeType = (MimeType)this.mimeType.clone();
            return clone;
        }
        return clone;
    }
    
    @Deprecated
    protected String normalizeMimeTypeParameter(String s, String s2) {
        return s2;
    }
    
    @Deprecated
    protected String normalizeMimeType(String s) {
        return s;
    }
    
    static {
        ioInputStreamClass = InputStream.class;
        stringFlavor = createConstant(String.class, "Unicode String");
        imageFlavor = createConstant("image/x-java-image; class=java.awt.Image", "Image");
        plainTextFlavor = createConstant("text/plain; charset=unicode; class=java.io.InputStream", "Plain Text");
        javaFileListFlavor = createConstant("application/x-java-file-list;class=java.util.List", null);
        DataFlavor.selectionHtmlFlavor = initHtmlDataFlavor("selection");
        DataFlavor.fragmentHtmlFlavor = initHtmlDataFlavor("fragment");
        DataFlavor.allHtmlFlavor = initHtmlDataFlavor("all");
    }
    
    static class TextFlavorComparator extends DataTransferer.DataFlavorComparator
    {
        public int compare(Object o, Object o2) {
            DataFlavor dataFlavor = (DataFlavor)o;
            DataFlavor dataFlavor2 = (DataFlavor)o2;
            if (dataFlavor.isFlavorTextType()) {
                if (dataFlavor2.isFlavorTextType()) {
                    return super.compare(o, o2);
                }
                return 1;
            }
            else {
                if (dataFlavor2.isFlavorTextType()) {
                    return -1;
                }
                return 0;
            }
        }
    }
}
